@@ -190,6 +190,9 @@ ${body}''')
             del self.content_type
             del self.content_length
 
+    def __str__(self):
+        return self.detail or self.explanation
+
     def _make_body(self, environ, escape):
         args = {
             'explanation': escape(self.explanation),
@@ -233,14 +236,20 @@ ${body}''')
         else:
             content_type = 'text/plain'
             body = self.plain_body(environ)
+        extra_kw = {}
+        if isinstance(body, unicode):
+            extra_kw.update(charset='utf-8')
         resp = Response(body,
             status=self.status,
             headerlist=headerlist,
-            content_type=content_type
+            content_type=content_type,
+            **extra_kw
         )
+        resp.content_type = content_type
         return resp(environ, start_response)
 
     def __call__(self, environ, start_response):
+        # FIXME: ensure HEAD and GET response headers are identical
         if environ['REQUEST_METHOD'] == 'HEAD':
             start_response(self.status, self.headerlist)
             return []
@@ -657,5 +666,6 @@ for name, value in globals().items():
         __all__.append(name)
         if getattr(value, 'code', None):
             status_map[value.code]=value
+        if hasattr(value, 'explanation'):
+            value.explanation = ' '.join(value.explanation.strip().split())
 del name, value
-
