@@ -6,21 +6,18 @@ Gives a multi-value dictionary object (MultiDict) plus several wrappers
 import cgi
 import copy
 import sys
-from webob.util.dictmixin import DictMixin
-try:
-    reversed
-except NameError:
-    from webob.util.reversed import reversed
+
+from webob.util import DictMixin, reversed
+
 
 __all__ = ['MultiDict', 'UnicodeMultiDict', 'NestedMultiDict', 'NoVars',
            'TrackableMultiDict']
 
 class MultiDict(DictMixin):
-
     """
-    An ordered dictionary that can have multiple values for each key.
-    Adds the methods getall, getone, mixed, and add to the normal
-    dictionary interface.
+        An ordered dictionary that can have multiple values for each key.
+        Adds the methods getall, getone, mixed, and add to the normal
+        dictionary interface.
     """
 
     def __init__(self, *args, **kw):
@@ -61,13 +58,12 @@ class MultiDict(DictMixin):
         Create a dict from a cgi.FieldStorage instance
         """
         obj = cls()
-        if fs.list:
-            # fs.list can be None when there's nothing to parse
-            for field in fs.list:
-                if field.filename:
-                    obj.add(field.name, field)
-                else:
-                    obj.add(field.name, field.value)
+        # fs.list can be None when there's nothing to parse
+        for field in fs.list or ():
+            if field.filename:
+                obj.add(field.name, field)
+            else:
+                obj.add(field.name, field.value)
         return obj
 
     from_fieldstorage = classmethod(from_fieldstorage)
@@ -138,16 +134,12 @@ class MultiDict(DictMixin):
 
     def dict_of_lists(self):
         """
-        Returns a dictionary where each key is associated with a
-        list of values.
+        Returns a dictionary where each key is associated with a list of values.
         """
-        result = {}
-        for key, value in self.iteritems():
-            if key in result:
-                result[key].append(value)
-            else:
-                result[key] = [value]
-        return result
+        r = {}
+        for key, val in self.iteritems():
+            r.setdefault(key, []).append(val)
+        return r
 
     def __delitem__(self, key):
         items = self._items
@@ -472,7 +464,7 @@ class NestedMultiDict(MultiDict):
     """
     Wraps several MultiDict objects, treating it as one large MultiDict
     """
-    
+
     def __init__(self, *dicts):
         self.dicts = dicts
 
@@ -553,7 +545,7 @@ class NestedMultiDict(MultiDict):
                 yield key
 
     iterkeys = __iter__
-    
+
 class NoVars(object):
     """
     Represents no variables; used when no variables
@@ -561,7 +553,7 @@ class NoVars(object):
 
     This is read-only
     """
-    
+
     def __init__(self, reason=None):
         self.reason = reason or 'N/A'
 
@@ -620,32 +612,3 @@ class NoVars(object):
     iteritems = iterkeys
     values = keys
     itervalues = iterkeys
-
-__test__ = {
-    'general': """
-    >>> d = MultiDict(a=1, b=2)
-    >>> d['a']
-    1
-    >>> d.getall('c')
-    []
-    >>> d.add('a', 2)
-    >>> d['a']
-    2
-    >>> d.getall('a')
-    [1, 2]
-    >>> d['b'] = 4
-    >>> d.getall('b')
-    [4]
-    >>> d.keys()
-    ['a', 'a', 'b']
-    >>> d.items()
-    [('a', 1), ('a', 2), ('b', 4)]
-    >>> d.mixed() == {'a': [1, 2], 'b': 4}
-    True
-    >>> MultiDict([('a', 'b')], c=2)
-    MultiDict([('a', 'b'), ('c', 2)])
-    """}
-
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
